@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_117:dys.h,v 1.21 2004/05/27 22:31:11 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_118:dys.h,v 1.22 2004/06/01 00:15:34 woods Exp"
  */
 
 /*
@@ -143,7 +143,7 @@
 /*
  * STR_FREE - free a region, returning its storage to the free pool
  */
-#define STR_FREE(sp)	(xfree((sp)->p), (sp)->p = NULL, (sp)->i = 0)
+#define STR_FREE(sp)	((sp)->p ? xfree((sp)->p) : 1, (sp)->p = NULL, (sp)->i = 0)
 
 /*
  * STR_ALIGN - if region index is not aligned add bytes to align it.
