@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_117:match.c,v 1.16 2004/05/30 15:52:10 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_118:match.c,v 1.18 2004/06/01 01:52:22 woods Exp"
  */
 
 /*
@@ -200,85 +200,124 @@ match_re_list(str, patterns, icase, reasonp)
     unsigned int icase;		/* ignore case? */
     char **reasonp;		/* optional pointer for returned message string */
 {
-    char *pat_p;
-    char *end_p;
-    size_t len;
+    size_t str_len = strlen(str);
+    char *ent_p;
+    char *end_ent;
     
-    if (reasonp) {
-	*reasonp = NULL;
-    }
+    *reasonp = NULL;
+
     if (!patterns) {
 	return MATCH_NOMATCH;
     }
 
     /* run through the pattern list */
-    for (pat_p = patterns; /* NOTEST */ ; pat_p = end_p + 1) {
+    for (ent_p = patterns; /* NOTEST */ ; ent_p = end_ent + 1) {
 	int matchresult = MATCH_MATCHED;
 	char prev_c;
 
-	/* skip any spaces at front */
-	while (*pat_p && isspace((int) *pat_p)) {
-	    pat_p++;
+	/* skip any spaces at the beginning */
+	while (*ent_p && isspace((int) *ent_p)) {
+	    ent_p++;
 	}
-	/* find the end of this pattern */
-	end_p = pat_p;
+	if (*ent_p == '\0') {
+	    DEBUG(DBG_MAIN_HI, "match_re_list(): skipping empty final entry.\n");
+	    break;
+	}
+	/*
+	 * find the end of this entry, i.e. the next unescaped ':'
+	 *
+	 * Note there are no other internal quoting mechanisms (as there are in
+	 * the full PCRE lists) except the backslash so we don't have to worry
+	 * about any other quoting delimiter.
+	 */
+	end_ent = ent_p;
 	prev_c = '\0';
-	while (*end_p && (*end_p != ':' || prev_c == '\\')) {
-	    prev_c = *end_p;
-	    end_p++;
+	while (*end_ent && (*end_ent != ':' || prev_c == '\\')) {
+	    prev_c = *end_ent;
+	    end_ent++;
 	}
-	/* check for a negation sign */
-	if (end_p > pat_p && *pat_p == '!') {
-	    pat_p++;
-	    matchresult = MATCH_NOMATCH;	/* a match of a negated item... */
-	    /* skip any spaces after the '!'  */
-	    while (*pat_p && pat_p > end_p && isspace((int) *pat_p)) {
-		pat_p++;
-	    }
-	}
-	/* skip empty patterns */
-	/* XXX should really allow empty patterns with "; text"  */
-	len = end_p - pat_p;
-	if (len > 0) {
-	    char *p;
-	    char *error;
+	/* only dig deeper into non-empty entries */
+	if (end_ent > ent_p) {
+	    char *start_re = ent_p;
+	    char *end_re = NULL;
 	    static struct str exp;
 	    static int exp_inited = FALSE;
 	    static struct str msg;
 	    static int msg_inited = FALSE;
 	    pcre_info_t *re_info;
+	    int result;
 
+	    /* check for a negation sign */
+	    if (*start_re == '!') {
+		start_re++;
+		matchresult = MATCH_NOMATCH;	/* a match of a negated item... */
+		/* skip any spaces after the '!'  */
+		while (*start_re && start_re > end_ent && isspace((int) *start_re)) {
+		    start_re++;
+		}
+	    }
+	    /*
+	     * search for the end of the RE, i.e. the next un-escaped ';', or
+	     * the end of the entry, or the end of the string, whichever comes
+	     * first
+	     */
+	    end_re = start_re;
+	    prev_c = '\0';
+	    while (*end_re &&
+		   (*end_re != ';' || prev_c == '\\') &&
+		   end_re < end_ent) {
+		prev_c = *end_re;
+		end_re++;
+	    }
+	    if (start_re == end_re) {
+		/* skip empty REs */
+		DEBUG(DBG_MAIN_HI, "match_re_list(): skipping empty RE\n");
+		continue;
+	    }
 	    if (!exp_inited) {
 		STR_INIT(&exp);
 		exp_inited = TRUE;
 	    } else {
+		STR_CHECK(&exp);
 		STR_CLEAR(&exp);
 	    }
-	    STR_NEXT(&exp, '^');		/* create a fully anchored RE pattern  */
-	    for (p = pat_p; len && *p != ';'; p++, len--) {
-		STR_NEXT(&exp, *p);
-	    }
-	    STR_CAT(&exp, "$");			/* NUL-terminates the string too */
+	    STR_NEXT(&exp, '^');	/* create a fully anchored RE pattern  */
+	    STR_NCAT(&exp, start_re, (size_t) (end_re - start_re));
+	    STR_CAT(&exp, "$");		/* NUL-terminates the string too */
+
 	    if (!msg_inited) {
 		STR_INIT(&msg);
 		msg_inited = TRUE;
 	    } else {
+		STR_CHECK(&msg);
 		STR_CLEAR(&msg);
 	    }
-	    /* note we're skipping the ';' itself */
-	    STR_NCAT(&msg, p + 1, len);
-	    STR_NEXT(&msg, '\0');
-
-	    if (!(re_info = compile_pcre_expr(STR(&exp), '/', icase ? "i" : "", STR(&msg), matchresult, &error))) {
-		/* XXX ignore returning error for now */
+	    if (*end_re == ';') {
+		/* note we're skipping the ';' itself */
+		STR_NCAT(&msg, end_re + 1, (size_t) (end_ent - end_re - 1));
+		STR_NEXT(&msg, '\0');
+	    }
+	    /* RE lists can be dynamic so we cannot cache this... */
+	    if (!(re_info = compile_pcre_expr(STR(&exp), '/', icase ? "i" : "", STR(&msg), matchresult, reasonp))) {
+		/* exp and msg are reused... */
 		return MATCH_FAIL;
 	    }
-	    switch (match_pcre_expr(str, strlen(str), re_info, TRUE, reasonp)) {
+	    result = match_pcre_expr(str, str_len, re_info, TRUE, reasonp);
+	    /* note re_info->{re,msg} are reused and re_info->opts is static */
+	    xfree((char *) re_info->code);
+	    xfree((char *) re_info);
+
+	    switch (result) {
 	    case MATCH_NOMATCH:
-		continue;
+		/* just fall through so the end-of-list test works! */
+		break;
 	    case MATCH_MATCHED:
+		DEBUG2(DBG_MAIN_MID, "match_re_list(): '%v' matched: returning %s\n",
+		       str,
+		       re_info->matchresult == MATCH_MATCHED ? "matched" : "NOT-matched");
 		return re_info->matchresult;
 	    case MATCH_FAIL:
+		DEBUG1(DBG_MAIN_MID, "match_re_list(): error: %s\n", *reasonp);
 		return MATCH_FAIL;
 #if 0 /* #ifndef NDEBUG */
 	    default:
@@ -286,8 +325,10 @@ match_re_list(str, patterns, icase, reasonp)
 #endif
 	    }
 	    /* continue with next pattern */
+	} else {
+	    DEBUG(DBG_MAIN_HI, "match_re_list(): skipping empty list entry\n");
 	}
-	if (*end_p == '\0') {
+	if (*end_ent == '\0') {
 	    break;				/* if there is one */
 	}
     }
@@ -368,7 +409,11 @@ match_ip(ip, patterns, fldsep, msgsep, allow_wildcard, reasonp)
 	while (*pat_p && isspace((int) *pat_p)) {
 	    pat_p++;
 	}
-	/* find the end of this pattern */
+	if (*pat_p == '\0') {
+	    DEBUG(DBG_MAIN_HI, "match_ip(): skipping empty final entry.\n");
+	    break;
+	}
+	/* find the end of this entry */
 	end_p = pat_p;
 	prev_c = '\0';
 	while (*end_p && (*end_p != fldsep || prev_c == '\\')) {
@@ -395,7 +440,8 @@ match_ip(ip, patterns, fldsep, msgsep, allow_wildcard, reasonp)
 		STR_INIT(&exp);
 		inited = TRUE;
 	    } else {
-		STR_LEN(&exp) = 0;
+		STR_CHECK(&exp);
+		STR_CLEAR(&exp);
 	    }
 	    STR_NCAT(&exp, pat_p, len);
 	    STR_NEXT(&exp, '\0');
@@ -642,9 +688,10 @@ compile_pcre_list(re_list, errorp)
     /*const*/ char *ent_p;
     /*const*/ char *end_ent;
 
+    *errorp = NULL;
+
     if (! re_list) {
-	*errorp = NULL;			/* not an error */
-	return nl;
+	return NULL;
     }
 
     /* run through the pattern list */
@@ -693,7 +740,7 @@ compile_pcre_list(re_list, errorp)
 	delim_c = *start_re++;
 	if (delim_c == ':') {
 	    DEBUG(DBG_MAIN_HI, "compile_pcre_list(): skipping empty entry.\n");
-	    continue;
+	    continue;			/* XXX is this always safe? */
 	}
 	if (delim_c == ';') {
 	    *errorp = xprintf("';' is not a valid RE delimiter");
@@ -735,6 +782,7 @@ compile_pcre_list(re_list, errorp)
 	if (start_re == end_re) {
 	    /* skip empty patterns */
 	    DEBUG(DBG_MAIN_HI, "compile_pcre_list(): skipping empty RE\n");
+	    /* this should be safe */
 	    continue;
 	}
 	STR_INIT(&exp);			/* do not re-use */
@@ -766,6 +814,9 @@ compile_pcre_list(re_list, errorp)
 	    STR_ZAP(&msg);		/* do not STR_FREE() any previous one! */
 	}
 	if (!(re_info = compile_pcre_expr(STR(&exp), delim_c, STR(&opts), STR(&msg), matchresult, errorp))) {
+	    STR_FREE(&exp);
+	    STR_FREE(&opts);
+	    STR_FREE(&msg);
 	    free_pcre_list(nl);
 	    return NULL;
 	}
@@ -868,13 +919,6 @@ compile_pcre_expr(exp, delim_c, opts, msg, matchresult, errorp)
 			  delim_c, re_info->re, delim_c,
 			  re_info->opts ? re_info->opts : "",
 			  err_offset, errtxt);
-	xfree((char *) re_info->re);
-	if (re_info->opts) {
-	    xfree((char *) re_info->opts);
-	}
-	if (re_info->msg) {
-	    xfree((char *) re_info->msg);
-	}
 	xfree((char *) re_info);
 	return NULL;
     }
@@ -888,14 +932,7 @@ compile_pcre_expr(exp, delim_c, opts, msg, matchresult, errorp)
 			  delim_c, re_info->re, delim_c,
 			  re_info->opts ? re_info->opts : "",
 			  errtxt);
-	xfree((char *) re_info->re);
 	xfree((char *) re_info->code);
-	if (re_info->opts) {
-	    xfree((char *) re_info->opts);
-	}
-	if (re_info->msg) {
-	    xfree((char *) re_info->msg);
-	}
 	xfree((char *) re_info);
 	return NULL;
     }
@@ -907,14 +944,7 @@ compile_pcre_expr(exp, delim_c, opts, msg, matchresult, errorp)
 			  rc,
 			  delim_c, re_info->re, delim_c,
 			  re_info->opts ? re_info->opts : "");
-	xfree((char *) re_info->re);
 	xfree((char *) re_info->code);
-	if (re_info->opts) {
-	    xfree((char *) re_info->opts);
-	}
-	if (re_info->msg) {
-	    xfree((char *) re_info->msg);
-	}
 	xfree((char *) re_info);
 	return NULL;
     }
