@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_117:smailconf.c,v 1.98 2004/05/15 00:10:48 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_118:smailconf.c,v 1.99 2004/06/01 00:17:19 woods Exp"
  */
 
 /*
@@ -1080,7 +1080,7 @@ read_standard_file(f, template, struct_size,
 	 */
 	/* FIXME:  avoid this ugly end_attr_table stuff -- use null-entry termination semantics... */
 	error = fill_attributes(cur, generic_attrs,
-				(long *) (cur + flags_offset),
+				(smail_bool_t *) (cur + flags_offset),
 				attr_table, end_attr_table, name);
 	if (error) {
 	    return xprintf("in entry %s", error); /* name is already included in error */
@@ -1261,7 +1261,7 @@ add_config_stat(fn, statp)
 	new->dev = statp->st_dev;
 	new->ino = statp->st_ino;
     } else {
-	new->mtime = (time_t)0;
+	new->mtime = 0;
     }
     new->succ = config_stat_list;
     config_stat_list = new;
