@@ -1,6 +1,6 @@
 #! /bin/sh
 :
-#ident	"@(#)smail/util:RELEASE-3_2_0_117:checkerr.sh,v 1.47 2004/05/26 06:13:18 woods Exp"
+#ident	"@(#)smail/util:RELEASE-3_2_0_118:checkerr.sh,v 1.48 2004/06/01 01:22:49 woods Exp"
 #
 #    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
 #    Copyright (C) 1992  Ronald S. Karr
@@ -432,103 +432,99 @@ mailq -s >> ${TMPDIR}/.chkerr.msg 2>&1
 	    sed 's|^error/||' | \
 	    sort | \
 	    comm -23 - .tempfail_bounces > .dead_bounces
-	if [ -s .dead_bounces ]; then
-	    rm -f .dead_bounce_senders
-	    : > .dead_bounce_senders
-	    rm -f .dead_double_bounces
-	    : > .dead_double_bounces
-	    while read bounce ; do
-
-		# Don't delete this message if it's not a
-		# double-bounce, i.e. if it doesn't have a new sender
-		# address of '<>' or '<+>'
-		#
-		# The sender address is the line right after the last
-		# '!-f', but we'll assume there's only ever one such
-		# line and so search only for the first one.
-		#
-		double_bounce="`sed -n -e '/^$/q' -e '1,/^!-f$/d;/<>/p;/<\+>/p;q' error/${bounce}`"
-
-		# If this message is a double-bounce then find the
-		# destination address for it is about to deleted.  The
-		# destination address is actually the sender address
-		# for the original un-deliverable message which cannot
-		# now be returned and is thus a double-bounce.  This
-		# list of sender addresses can be used to reject
-		# future e-mails from these obviously bogus or broken
-		# sender addresses.
-		#
-		# We assume there's only ever one recipient for a bounce.
-		#
-		if [ -n "${double_bounce}" ]; then
-		    echo ${bounce} >> .dead_double_bounces
-		    mailq -E ${bounce} | \
-			sed -n '/^		Rcpts: /s///p' >> .dead_bounce_senders
-		fi
-	    done < .dead_bounces
-	    
-	    # add a nice comment if it'll be a new dead-mail.senders file
-	    #
-	    if [ ! -s ${SMAIL_LIB_DIR}/dead-mail.senders ]; then
-		(
-		    echo "#"
-		    echo "# NOTICE: entries are automatically appended to this file by '$argv0'"
-		    echo "#"
-		    echo "# you can point smtp_sender_reject_db at this file"
-		    echo "#"
-		    echo "# old entries, e.g. ones prior to 2003/01/21, could be"
-		    echo "# removed from this file using a filter like this:"
-		    echo "#"
-		    echo "#  awk '\$1 ~ /^#/ || \$4 > 20030121 { print \$0;}' "
-		    echo "#"
-		) > ${SMAIL_LIB_DIR}/dead-mail.senders
+	if [ ! -s .dead_bounces ]; then
+	    if ${very_verbose}; then
+		echo "No frozen double-bounced message matched the patterns" >> .checkerror
+		echo "in ${DEAD_MAIL_ERE_FILE}" >> .checkerror
+		echo "" >> .checkerror
 	    fi
-
-	    if [ -s .dead_bounce_senders ]; then
-
-		# take the unique sender addresses from the list created
-		# above and add them to the dead-mail.senders file in a
-		# format suitable for use with lsearch_database()
-		#
-		DATE_ADDED=`date +%Y%m%d`
-		sort -u .dead_bounce_senders | \
-		    sed -e 's/^<//' \
-			-e 's/>$//' \
-			-e 's/$/: added on '${DATE_ADDED}' due to failed double-bounce/'
-	    fi >> ${SMAIL_LIB_DIR}/dead-mail.senders
-		
-	    if [ -s .dead_double_bounces ]; then
-
-		# The .dead_bounces messages which are not listed in
-		# .dead_double_bounces are probably mis-matches.  We'll
-		# ignore them for now, but perhaps they should be noted as
-		# such so that the dead-mail.egrep patterns can be made
-		# more accurate if possible.
-		#
-		# On the other hand these messages could be bounces
-		# being relayed through this host as a smart-host, and
-		# now we're stuck with them.  In this case the best
-		# thing to do is simply remove them, but of course
-		# that can only be done after careful manual
-		# examination.
-		#
-		rm -f "" `sed 's|^|error/|' .dead_double_bounces`
-		rm -f "" `sed 's|^|msglog/|' .dead_double_bounces`
-		
-		echo "Removed" `wc -l < .dead_double_bounces` "frozen and double-bounced messages matching"
-		echo "the patterns in ${DEAD_MAIL_ERE_FILE}!"
-		echo ""
-		echo "The list of corresponding message-IDs for these removed bounces remains in"
-		echo `pwd`/.dead_double_bounces "until the next run of '$argv0'."
-		echo ""
-	    fi >> .checkerror
-	elif ${very_verbose}; then
-	    echo "No frozen double-bounced message matched the patterns"
-	    echo "in ${DEAD_MAIL_ERE_FILE}" >> .checkerror
-	    echo "" >> .checkerror
 	fi
     fi
 
+    rm -f .dead_bounce_senders
+    : > .dead_bounce_senders
+
+    # now again for the ERE-only matches
+    #
+    # NOTE: keep in sync with below -- XXX should be a function....
+    #
+    if [ -s .dead_bounces ]; then
+	rm -f .dead_double_bounces
+	: > .dead_double_bounces
+	while read bounce ; do
+	    
+	    # Don't delete this message if it's not a
+	    # double-bounce, i.e. if it doesn't have a new sender
+	    # address of '<>' or '<+>'
+	    #
+	    # The sender address is the line right after the last
+	    # '!-f', but we'll assume there's only ever one such
+	    # line and so search only for the first one.
+	    #
+	    double_bounce="`sed -n -e '/^$/q' -e '1,/^!-f$/d;/<>/p;/<\+>/p;q' error/${bounce}`"
+
+	    # If this message is a double-bounce then find the
+	    # destination address for it is about to deleted.  The
+	    # destination address is actually the sender address
+	    # for the original un-deliverable message which cannot
+	    # now be returned and is thus a double-bounce.  This
+	    # list of sender addresses can be used to reject
+	    # future e-mails from these obviously bogus or broken
+	    # sender addresses.
+	    #
+	    # We assume there's only ever one recipient for a bounce.
+	    #
+	    if [ -n "${double_bounce}" ]; then
+		echo ${bounce} >> .dead_double_bounces
+		mailq -E ${bounce} | \
+		    sed -n '/^		Rcpts: /s///p' >> .dead_bounce_senders
+	    fi
+	done < .dead_bounces
+    fi
+
+    # add a nice comment if it'll be a new dead-mail.senders file
+    #
+    if [ ! -s ${SMAIL_LIB_DIR}/dead-mail.senders ]; then
+	(
+	    echo "#"
+	    echo "# NOTICE: entries are automatically appended to this file by '$argv0'"
+	    echo "#"
+	    echo "# you can point smtp_sender_reject_db at this file"
+	    echo "#"
+	    echo "# old entries, e.g. ones prior to 2003/01/21, could be"
+	    echo "# removed from this file using a filter like this:"
+	    echo "#"
+	    echo "#  awk '\$1 ~ /^#/ || \$4 > 20030121 { print \$0;}' "
+	    echo "#"
+	) > ${SMAIL_LIB_DIR}/dead-mail.senders
+    fi
+
+    if [ -s .dead_double_bounces ]; then
+
+	# The .dead_bounces messages which are not listed in
+	# .dead_double_bounces are probably mis-matches.  We'll
+	# ignore them for now, but perhaps they should be noted as
+	# such so that the dead-mail.egrep patterns can be made
+	# more accurate if possible.
+	#
+	# On the other hand these messages could be bounces
+	# being relayed through this host as a smart-host, and
+	# now we're stuck with them.  In this case the best
+	# thing to do is simply remove them, but of course
+	# that can only be done after careful manual
+	# examination.
+	#
+	rm -f "" `sed 's|^|error/|' .dead_double_bounces`
+	rm -f "" `sed 's|^|msglog/|' .dead_double_bounces`
+		
+	echo "Removed" `wc -l < .dead_double_bounces` "frozen and double-bounced messages matching"
+	echo "the patterns in ${DEAD_MAIL_ERE_FILE}!"
+	echo ""
+	echo "The list of corresponding message-IDs for these removed bounces remains in"
+	echo `pwd`/.dead_double_bounces "until the next run of '$argv0'."
+	echo ""
+    fi >> .checkerror
+
     # find any new errors and report any related information....
     #
     if [ -f .lasttimedone ]; then
@@ -563,12 +559,45 @@ p;
 :quit
 q' error/$f`"
 	if [ -n "$frozen_sender" ] ; then
+	    echo ""
 	    echo "WARNING: This message was likely frozen because of some local system error"
 	    echo "or Smail configuration error!  (it is not a bounce message)"
 	    echo ""
 	fi
 
-	# next we include a full status of the message
+	# detect anything that is likely a dead double-bounce
+	# and add any non-local recipient to the .dead_bounce_senders list
+	#
+	# note: for now we will do this explicitly instead of just
+	# using [ -z "$frozen_sender" ]
+	#
+	if [ -n "`sed -n -e '/^$/q' -e '1,/^!-f$/d;/<>/p;/<\+>/p;q' error/$f`" ]; then
+
+	    # record undeliverable addresses from double-bounces for
+	    # smtp_reject_sender_db
+	    #
+	    # Note these are not deleted -- they still need manual attention.
+	    #
+	    sender="`mailq -E $f | sed -n '/^		Rcpts: /s///p'`"
+
+	    # This is a really nasty hack but it's the best I can think of!
+	    #
+	    addrinfo="`$SMAIL_PROGRAM -v -bv ${sender} 2>&1`"
+	    count=`expr "${addrinfo}" : '.*director .*')`
+	    echo ""
+	    if [ $count -gt 0 ]; then
+		echo "WARNING: This message is a double bounce from a local original sender."
+		echo "The address <${sender}> was not deliverable."
+		echo "This message should be unfrozen when the problem has been resolved."
+	    else
+		echo "Note: This message is a double bounce from a non-local original sender."
+		echo "Logging sender address <${sender}> as invalid."
+		echo $sender >> .dead_bounce_senders
+	    fi
+	    echo ""
+	fi
+
+	# next we include a full status of the message in .checkerror
 	#
 	mailq -E -v $f
 
@@ -639,6 +668,20 @@ q' error/$f`"
 	rm .tmp_paniclog
     done >> .checkerror
 
+    if [ -s .dead_bounce_senders ]; then
+
+	# take the unique sender addresses from the combined list
+	# created above and add them to the dead-mail.senders file in
+	# a format suitable for use with lsearch_database()
+	# which will be pointed to by default by smtp_reject_senders_db
+	#
+	DATE_ADDED=`date +%Y%m%d`
+	sort -u .dead_bounce_senders | \
+	    sed -e 's/^<//' \
+	    -e 's/>$//' \
+	    -e 's/$/: added on '${DATE_ADDED}' due to failed double-bounce/'
+    fi >> ${SMAIL_LIB_DIR}/dead-mail.senders
+		
     if [ -s .checkerror -o -s ${TMPDIR}/.chkerr.msg ]; then
 	# don't try to send to the Postmaster if configuration errors still exist
 	if ${SMAIL_PROGRAM} -bv Postmaster > /dev/null 2>&1 ; then
